function [pars,fval,exitflag,output] = fminralg(varargin)
% [MINIMUM,FVAL,EXITFLAG,OUTPUT] = FMINRALG(FUN,PARS,[OPTIONS],[CONSTRAINTS]) Shor's r-algorithm minimization
%
% This minimization method uses the Shor's r-algorithm method, and only works with
% more than one parameter to optimize.
% 
% Calling:
%   fminralg(fun, pars) asks to minimize the 'fun' objective function with starting
%     parameters 'pars' (vector)
%   fminralg(fun, pars, options) same as above, with customized options (optimset)
%   fminralg(fun, pars, options, fixed) 
%     is used to fix some of the parameters. The 'fixed' vector is then 0 for
%     free parameters, and 1 otherwise.
%   fminralg(fun, pars, options, lb, ub) 
%     is used to set the minimal and maximal parameter bounds, as vectors.
%   fminralg(fun, pars, options, constraints) 
%     where constraints is a structure (see below).
%   fminralg(problem) where problem is a structure with fields
%     problem.objective:   function to minimize
%     problem.x0:          starting parameter values
%     problem.options:     optimizer options (see below)
%     problem.constraints: optimization constraints
%
% Example:
%   banana = @(x)100*(x(2)-x(1)^2)^2+(1-x(1))^2;
%   [x,fval] = fminralg(banana,[-1.2, 1])
%
% Input:
%  FUN is the function to minimize (handle or string).
%
%  PARS is a vector with initial guess parameters. You must input an
%  initial guess. Dimensionality must be greater than 1.
%
%  OPTIONS is a structure with settings for the optimizer, 
%  compliant with optimset. Default options may be obtained with
%      o=fminralg('defaults')
%
%  CONSTRAINTS may be specified as a structure
%   constraints.min=   vector of minimal values for parameters
%   constraints.max=   vector of maximal values for parameters
%   constraints.fixed= vector having 0 where parameters are free, 1 otherwise
%   constraints.step=  vector of maximal parameter changes per iteration
%
% Output:
%          MINIMUM is the solution which generated the smallest encountered
%            value when input into FUN.
%          FVAL is the value of the FUN function evaluated at MINIMUM.
%          EXITFLAG return state of the optimizer
%          OUTPUT additional information returned as a structure.
%
% Reference: Shor N.Z., Minimization Methods for Non-Differentiable Functions,
%   Springer Series in Computational Mathematics, Vol. 3, Springer-Verlag (1985)
%
% Contrib: Alexei Kuntsevich alex@bedvgm.kfunigraz.ac.at 
%   and Franz Kappel franz.kappel@kfunigraz.ac.at, Graz (Austria) 1997 [solvopt]
%
% Version: $Revision: 1.14 $
% See also: fminsearch, optimset

% default options for optimset
if nargin == 0 || (nargin == 1 && strcmp(varargin{1},'defaults'))
  options=optimset; % empty structure
  options.Display='';
  options.TolFun =1e-3;
  options.TolX   =1e-8;
  options.MaxIter=1000;
  options.MaxFunEvals=5000;
  options.algorithm  = [ 'Shor r-algorithm (by Kuntsevich) [' mfilename ']' ];
  options.optimizer = mfilename;
  pars = options;
  return
end

[pars,fval,exitflag,output] = fmin_private_wrapper(mfilename, varargin{:});

